'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

var splitSdkProvider = require('@personio-internal/split-sdk-provider');

var React = require('react');

function _interopDefaultLegacy(e) {
  return e && typeof e === 'object' && 'default' in e ? e : {
    'default': e
  };
}

var React__default = /*#__PURE__*/_interopDefaultLegacy(React);

var DYNAMIC_COMPONENT_MARK = 'function () { return Promise.resolve().then(';
exports.FeatureFlag = void 0;

(function (FeatureFlag) {
  FeatureFlag["EnableFlowVisualLanguage"] = "DST-enable-flow-visual-language";
})(exports.FeatureFlag || (exports.FeatureFlag = {}));

var HUI_CLASSNAME_PREFIX = 'hui-visual-language';
var visualLanguageClassNames = {
  Vintage: HUI_CLASSNAME_PREFIX + "_vintage",
  Flow: HUI_CLASSNAME_PREFIX + "_flow"
}; // Keep a map of all feature flags that we want
// to persist across multiple calls of the hook.

var persistedFlags = {};

function usePersistedFeatureFlag(flag) {
  var featureFlag = splitSdkProvider.useFeatureFlag(flag);
  var persistedFlag = persistedFlags[flag];
  if (persistedFlag !== undefined) return persistedFlag; // Only persist a flag if it has been loaded.
  // This prevents storing a flag that has not
  // resolved its initial value.

  if (featureFlag.isReady) {
    persistedFlags[flag] = featureFlag;
  }

  return featureFlag;
}

function useVisualLanguage(_a) {
  var _b = _a === void 0 ? {} : _a,
      _c = _b.visualLanguage,
      desiredVisualLanguage = _c === void 0 ? 'Flow' : _c,
      _d = _b.flag,
      flagName = _d === void 0 ? exports.FeatureFlag.EnableFlowVisualLanguage : _d,
      _e = _b.force,
      forceVisualLanguage = _e === void 0 ? false : _e; // Handle the special case when Flow visual language is forced
  // through a global feature flag.


  var globalFlowEnabled = usePersistedFeatureFlag(exports.FeatureFlag.EnableFlowVisualLanguage).isOn; // Don't allow the visual language to change after the flag
  // is set once.

  var specifiedFeatureFlagEnabled = usePersistedFeatureFlag(flagName).isOn; // Global Flow enablement trumps all others, followed by the
  // forcing of the visual language, and finally by the specified
  // feature flag value. If none of these are true, then the
  // visual language is Vintage.
  // We need to track both the visual language and whether the
  // specified visual language is enabled. It is possible for a
  // visual language classname to be returned that does not match
  // the visual language passed in.

  var appliedVisualLanguage;
  var desiredVisualLanguageEnabled;

  if (globalFlowEnabled) {
    appliedVisualLanguage = 'Flow';
    desiredVisualLanguageEnabled = desiredVisualLanguage === 'Flow';
  } else if (forceVisualLanguage || specifiedFeatureFlagEnabled) {
    appliedVisualLanguage = desiredVisualLanguage;
    desiredVisualLanguageEnabled = true;
  } else {
    appliedVisualLanguage = 'Vintage'; // It is possible that the specified visual language is Vintage
    // but the specified feature flag is not turned on. In this case,
    // we still want to treat the visual language as being enabled.

    desiredVisualLanguageEnabled = desiredVisualLanguage === 'Vintage';
  }

  var className = visualLanguageClassNames[appliedVisualLanguage];
  return {
    enabled: desiredVisualLanguageEnabled,
    className: className
  };
}
/* eslint-disable import/prefer-default-export */


var isDynamicComponent = function (component) {
  var str = String(component);
  return str.startsWith(DYNAMIC_COMPONENT_MARK);
};

function VisualLanguageSwitch(_a) {
  var on = _a.on,
      off = _a.off,
      _b = _a.flag,
      flag = _b === void 0 ? exports.FeatureFlag.EnableFlowVisualLanguage : _b,
      _c = _a.visualLanguage,
      visualLanguage = _c === void 0 ? 'Flow' : _c,
      _d = _a.force,
      force = _d === void 0 ? false : _d,
      _e = _a.fallback,
      fallback = _e === void 0 ? null : _e; // Use the hook to handle the multiple factors that go into
  // determining whether the visual language is enabled

  var desiredVisualLanguageEnabled = useVisualLanguage({
    visualLanguage: visualLanguage,
    flag: flag,
    force: force
  }).enabled; // Only render the fallback if the desired language is not already
  // enabled and we don't know if the feature flag will enable it

  var isReady = splitSdkProvider.useFeatureFlag(flag).isReady;
  if (!isReady && !desiredVisualLanguageEnabled) return React__default["default"].createElement(React__default["default"].Fragment, null, fallback);
  var winner = desiredVisualLanguageEnabled ? on : off;
  if (!isDynamicComponent(winner)) return React__default["default"].createElement(React__default["default"].Fragment, null, winner);
  var LazyComponent = React__default["default"].lazy(winner);
  return React__default["default"].createElement(React.Suspense, {
    fallback: fallback
  }, React__default["default"].createElement(LazyComponent, null));
}

exports.DYNAMIC_COMPONENT_MARK = DYNAMIC_COMPONENT_MARK;
exports.VisualLanguageSwitch = VisualLanguageSwitch;
exports.useVisualLanguage = useVisualLanguage;
exports.visualLanguageClassNames = visualLanguageClassNames;
//# sourceMappingURL=index.js.map
